import pygame as pg


class GUI:
    """Class for the GUI object."""
    def __init__(self):
        """Initializes the GUI object."""
        self.font_path = "assets/fonts/Moon_Dance/MoonDance-Regular.ttf"
        self.font_color = (255, 255, 255)
        self.font_size = 64
        self.font = pg.font.Font(self.font_path, self.font_size)
        self.margin = 32

    def render_text(self, text: str) -> pg.Surface:
        """
        Renders the text and returns it on a surface.

        Parameters:
            text:
                The text to render.

        Returns:
            rendered_text:
                The rendered text.
        """
        rendered_text = self.font.render(text, True, self.font_color)
        return rendered_text

    def draw_text(self, surface, text: str, position: tuple[float, float], anchor: str = 'topleft', font_size: int = None):
        """
        Draws the text to the passed surface.

        Parameters:
            surface:
                The surface to draw the text to.
            text:
                The text to draw.
            position:
                The position of the text.
            anchor:
                The anchor point of the text.
            font_size:
                The font size for rendering (optional).
        """
        if font_size:
            font = pg.font.Font(self.font_path, font_size)
        else:
            font = self.font

        rendered_text = font.render(text, True, self.font_color)
        text_rect = rendered_text.get_rect()
        setattr(text_rect, anchor, position)
        surface.blit(rendered_text, text_rect)


    def show_end_screen(self, window, score):
        window.fill((0, 0, 0))  # Background

        # For each round went throught you get 100 score >:)
        self.draw_text(window, "GAME OVER", (window.get_width() // 2, window.get_height() // 3), 'center', font_size=48)
        self.draw_text(window, f"Final Score: {score}", (window.get_width() // 2, window.get_height() // 2), 'center', font_size=36)

        # Final Screen
        self.draw_text(window, "ESC to quit", (window.get_width() // 2, window.get_height() * 2 // 3), 'center', font_size=24)

        pg.display.update()

        waiting_for_input = True
        while waiting_for_input:
            for event in pg.event.get():
                if event.type == pg.QUIT or (event.type == pg.KEYDOWN and event.key == pg.K_ESCAPE):
                    pg.quit()
                    exit()
