########################################################################
# Programmeerimine I
# 2024/2025 sügissemester
#
# Projekt Zavi
# Teema: Torni kaitse mäng
#
#
# Autorid: Mihkel Hain, Jan Erik Jakstein
#
# Mõningane eeskuju: Tower Defense mängud, nagu Bloons Tower Defense
#
# Soovitus: Soovitame kasutada pygame-ce
#
########################################################################

import pygame as pg
import time
from world import World
from gui import GUI
import sound

def main():
    # World object creation
    world = World()

    # GUI object creation
    gui = GUI()

    dt = 1/1000
    time_now = time.time()
    #MOUSE TESTING
    mouseSquare = pg.Rect(0,0,100,100)
    score = 0

    # Main game loop
    running = True
    while running:
        # Calculate delta time
        dt = time.time() - time_now
        time_now = time.time()

        # Input handling
        for event in pg.event.get():
            if event.type == pg.QUIT:
                running = False
            # ESC key exits the game
            if event.type == pg.KEYDOWN:
                if event.key == pg.K_ESCAPE:
                    running = False
                # Choose the tower to place with the number keys
                if event.key == pg.K_1:
                    if world.active_tower != 1:
                        world.active_tower = 1
                    else:
                        world.active_tower = 0
                if event.key == pg.K_2:
                    if world.active_tower != 2:
                        world.active_tower = 2
                    else:
                        world.active_tower = 0
                # Start the round with the space key
                if event.key == pg.K_SPACE and not world.round_active:
                    world.request_to_start_round = True

            #If right mouse button pressed calls the function to create the turret
            if event.type == pg.MOUSEBUTTONDOWN:
                mouse_pos = pg.mouse.get_pos()
                world.turret_spawn(mouse_pos)

        # Game state updates
        if world.round_active:

            # Spawn enemies at the correct time
            for enemy in world.round_spawn_times:
                if time.time() - world.round_start_time > enemy[1]:
                    world.enemies.append(enemy[0])
                    world.round_spawn_times.pop(0)
                    break

            for enemy in world.enemies[:]:
                if enemy.move_along_path(dt, world.waypoints):
                    # Enemy has reached the end of the path
                    world.enemies.remove(enemy)
                    world.player_health -= 10

                    #If the player is dead
                    if world.player_health <= 0:
                        world.round_active = False
                        gui.show_end_screen(world.window,score)
                        running = False


            # Turret logic
            world.turret_attacks(dt)

            # If spawn list is empty and no enemies left, end the round
            if world.round_active and len(world.round_spawn_times) == 0 and len(world.enemies) == 0:
                world.round_active = False
                sound.round_win.play()
                score += 100


        if world.request_to_start_round:
            world.current_round += 1
            world.round_spawn_times = world.generate_enemy_spawn_times(world.current_round)
            world.request_to_start_round = False
            world.round_active = True
            world.round_start_time = time.time()


        # Drawing the world and it's objects
        world.draw()

        #Box for collision tower purposes
        if world.active_tower != 0:
            mouseSquare.center = pg.mouse.get_pos()
            pg.draw.rect(world.window, (255, 0, 0), mouseSquare, 6, 1)

        # GUI drawing
        # Draw the round number
        gui.draw_text(world.window, f"ROUND: {world.current_round}", (world.window_resolution[0] - gui.margin, gui.margin), 'topright')
        # Draw money balance
        gui.draw_text(world.window, f"MONEY: {world.money}", (world.window_resolution[0] - gui.margin, gui.margin * 2 + gui.font_size), 'topright')
        # Draw the player health at the top middle of the screen
        gui.draw_text(world.window, f"HEALTH: {world.player_health}", (world.window_resolution[0] // 2, gui.margin), 'midtop')
        # Debug drawing, comment out if not needed.
        #world.draw_debug()

        pg.display.update()

    pg.quit()

if __name__ == "__main__":
    main()
