import pygame as pg
from enemy import Enemy
import math

class Turret(pg.sprite.Sprite):
    def __init__(self, image, pos):
        pg.sprite.Sprite.__init__(self)
        self.texture = pg.image.load('assets/images/turrets/red_turret_128x128.png').convert_alpha()
        self.rotated_texture = self.texture
        self.rect = self.texture.get_rect()
        self.rotated_rect = self.rect
        self.rect.center = pos

        # Turret stats
        self.cost = 500
        self.range = 270
        self.attack_damage = 20
        self.attack_cooldown = 1
        self.time_since_last_attack = self.attack_cooldown

    def find_nearest_enemy(self, enemies: list[Enemy]) -> Enemy:
        """
        Returns the nearest enemy object within range.
        """
        nearest_enemy = None
        nearest_distance = self.range  # Only consider enemies within the range
        for enemy in enemies:
            distance = self.distance_to_enemy(enemy)
            if distance < nearest_distance:
                nearest_enemy = enemy
                nearest_distance = distance
        return nearest_enemy

    def distance_to_enemy(self, enemy):
        """
        Calculates the distance between the turret and the given enemy.
        """
        return math.hypot(enemy.screen_position[0] - self.rect.center[0],
                          enemy.screen_position[1] - self.rect.center[1])

    def face_at_the_current_target(self, target: Enemy):
        """
        Rotates the turret to face the target enemy.
        """
        dx = target.screen_position[0] - self.rect.center[0]
        dy = target.screen_position[1] - self.rect.center[1]
        angle = math.degrees(math.atan2(-dy, dx))
        self.rotated_texture = pg.transform.rotozoom(self.texture, angle, 1)
        self.rotated_rect = self.rotated_texture.get_rect(center=self.rect.center)

    #This draws the range
    def draw(self, surface):
        range_color = (255, 0, 0, 25)
        transparent_surface = pg.Surface((self.range * 2, self.range * 2), pg.SRCALPHA)
        pg.draw.circle(transparent_surface, range_color, (self.range, self.range), self.range)
        surface.blit(transparent_surface, (self.rect.center[0] - self.range, self.rect.center[1] - self.range))

        # Draw the turret
        surface.blit(self.rotated_texture, self.rotated_rect)

    def update(self, enemies):
        """
        Updates the turret's logic: targets and rotates towards enemies.
        """
        nearest_enemy = self.find_nearest_enemy(enemies)
        if nearest_enemy:
            self.face_at_the_current_target(nearest_enemy)
